package com.winhc.phoenix.example.controller;

import com.aliyun.odps.utils.StringUtils;
import com.winhc.phoenix.example.aspect.Timer;
import com.winhc.phoenix.example.enums.EsVersion;
import com.winhc.phoenix.example.service.SearchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;
import java.util.regex.Pattern;

/**
 * @author: XuJiakai
 * 2020/11/19 14:51
 */
@Slf4j
@AllArgsConstructor
@RestController
@Api(tags = "es查询", value = "es")
@RequestMapping("es/{version}")
public class SearchController {
    private final Map<String, SearchService> map;

    @Timer
    @ApiOperation(value = "es搜索")
    @GetMapping("query")
    public Object query(String content, @RequestParam(defaultValue = "v8_simp") EsVersion version, @RequestParam(defaultValue = "0") int from, @RequestParam(defaultValue = "10") int size) {
        return map.get(version.getValue()).query(cleanup(content), from, size);
    }


    @Timer
    @ApiOperation(value = "搜索对照组")
    @GetMapping("control")
    public Object controlGroup(String content, @RequestParam(defaultValue = "v8_simp") EsVersion version) {
        return map.get(version.getValue()).controlGroup(cleanup(content));
    }

    @Timer
    @ApiOperation(value = "搜索提示（测试中）")
    @GetMapping("tips")
    public Object searchTips(String content, @RequestParam(defaultValue = "v8_simp") EsVersion version) {
        return map.get(version.getValue()).tips(cleanup(content));
    }


    private static final Pattern pattern = Pattern.compile("[^\\u4e00-\\u9fa50-9a-zA-Z]");

    private static String cleanup(String val) {
        return StringUtils.isNotBlank(val) ? pattern.matcher(val).replaceAll("") : "";
    }
}
