package com.winhc.phoenix.example.dao.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.winhc.phoenix.example.dao.SearchDao;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.springframework.stereotype.Repository;

import java.util.HashMap;

/**
 * @author: XuJiakai
 * 2020/11/20 15:16
 */
@Slf4j
@Repository
@AllArgsConstructor
public class SearchDaoImpl implements SearchDao {
    private RestHighLevelClient restHighLevelClient;


    private ObjectMapper mapper;
    private static final TypeReference<HashMap<String, Object>> typeRef
            = new TypeReference<HashMap<String, Object>>() {
    };

    @SneakyThrows
    @Override
    public Object search(String index, String type, QueryBuilder query, SortBuilder sortBuilder, FetchSourceContext fetchSourceContext, int from, int size) {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder()
                .query(query)
                .from(from)
                .size(size)
                .highlighter(new HighlightBuilder().field("*").preTags("<font color='red'>").postTags("</font>"));

        if (fetchSourceContext != null) {
            searchSourceBuilder.fetchSource(fetchSourceContext);
        }
        if (sortBuilder != null) {
            searchSourceBuilder.sort(sortBuilder).trackScores(true);
        }

        SearchRequest searchRequest = new SearchRequest()
                .indices(index)
                .types(type)
                .source(searchSourceBuilder);

        SearchResponse search = restHighLevelClient.search(searchRequest);
        return mapper.readValue(search.toString(), typeRef);
    }


    private final RestClient restClient;

    @SneakyThrows
    public Object test() {
        Response get = restClient.performRequest("get", "_cat/indices/judicial_case_v*?h=index");
        String s = EntityUtils.toString(get.getEntity());
        System.out.println(s);

        return null;
    }
}
